/*
 * Copyright (C) 2006-2007 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

/** \file write.h writer-related structures */

/** the list of supported writers */
enum writer_enum {WRITER_ENUM};

/**
 * decbribes one running instance of a writer
 */
struct writer_node {
	/** points to the writer structure associated with this node */
	struct writer *writer; /* FIXME: Should better be only the number */
	/** writer-specific data */
	void *private_data;
	/** send that many bytes in one go */
	int chunk_bytes;
	/** pointer to the group this node belongs to */
	struct writer_node_group *wng;
	/** the writer-specific configuration of this node */
	void *conf;
	/** how much of the wng's buffer is already written */
	size_t written;
};

/** describes one supported writer */
struct writer {
	/**
	 * the init function of the writer
	 *
	 * It must fill in all other function pointers of the given
	 * writer structure.
	 *
	 */
	void (*init)(struct writer *w);
	/**
	 *
	 *
	 * the command line parser of the writer
	 *
	 * It should check whether the command line options given by \a options are
	 * valid.  On success, it should return a pointer to the writer-specific
	 * configuration data determined by \a options.  Note that this might be called
	 * more than once with different values of \a options.
	 *
	 */
		void * (*parse_config)(const char *options);
	/**
	 *
	 * open one instance of this writer
	 *
	 * This function should perform any work necessary to write the incoming
	 * stream. If To this aim, it may allocate its private data structure and store
	 * a pointer to that structure via the given writer_node paramenter.
	 */
	int (*open)(struct writer_node *);
	/**
	 *
	 * write a chunk of audio data
	 *
	 * This is called from the writer node group task's pre_select(). It
	 * may use the sched pointer to add any file descriptors or to decrease
	 * the select timeout. It must return positive on success and negative
	 * on errors.
	 */
	int (*pre_select)(struct sched *s, struct writer_node *wn);
	/**
	 * Called from the post_select function of the wng task. It must keep
	 * track of the the number of bytes consumed from the wng's buffer via
	 * the wn->written variable (which may be modified by the wng handling
	 * functions). This function must return positive on success and
	 * negative on errors.
	 */
	int (*post_select)(struct sched *s, struct writer_node *wn);
	/**
	 * close one instance of the writer
	 *
	 * This function is assumed to succeed.
	 */
	void (*close)(struct writer_node *);
	/**
	 * shutdown the writer
	 *
	 * This is a optional function pointer used for cleaning
	 * up.
	 */
	void (*shutdown)(struct writer_node *);
};

/**
 * describes a set of writer nodes that all write the same stream.
 */
struct writer_node_group {
	/** number of nodes belonging to this group */
	unsigned num_writers;
	/** array of pointers to the corresponding writer nodes */
	struct writer_node *writer_nodes;
	/** the maximum of the chunk_bytes values of the writer nodes in this group */
	size_t max_chunk_bytes;
	/** non-zero if end of file was encountered by the feeding task */
	int *input_eof;
	/** non-zero if end of file was encountered */
	int eof;
	/** current output buffer */
	char *buf;
	/** number of bytes loaded in the output buffer */
	size_t *loaded;
	/** number of audio channels of the current stream */
	unsigned int *channels;
	/** sample rate of the current stream */
	unsigned int *samplerate;
	/** the task associated to this group */
	struct task task;
};

/** loop over each writer node in a writer group */
#define FOR_EACH_WRITER_NODE(i, wng) for (i = 0; i < (wng)->num_writers; i++)
/** loop over each supported writer */
#define FOR_EACH_WRITER(i) for (i = 0; i < NUM_SUPPORTED_WRITERS; i++)

/** declare the init functions of all supported writers */
DECLARE_WRITER_INITS;

/** array containing the name of each writer */
extern const char *writer_names[];

/** the writer structure for each supported writer */
extern struct writer writers[NUM_SUPPORTED_WRITERS];
